'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _makeAst = require('./make-ast.js');

var _makeAst2 = _interopRequireDefault(_makeAst);

var _jsonPointer = require('json-pointer');

var _jsonPointer2 = _interopRequireDefault(_jsonPointer);

var _ast = require('./ast/ast.js');

var _ast2 = _interopRequireDefault(_ast);

var _util = require('../util.js');

var _util2 = _interopRequireDefault(_util);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var generateFlowAsts = function generateFlowAsts(schema) {
  var shape = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : { root: schema };

  var gensym = _util2.default.gengensym();

  var cache = new WeakMap(_lodash2.default.map(shape, function (subSchema, name) {
    return [subSchema, name];
  }));
  var schemas = _lodash2.default.values(shape);

  var symbolForSchema = function symbolForSchema(schm) {
    if (!cache.has(schm)) {
      cache.set(schm, gensym('T'));
      schemas.push(schm);
    }
    return cache.get(schm);
  };

  var refResolver = function refResolver(ref) {
    if (ref.startsWith('#')) {
      var subSchema = _jsonPointer2.default.get(schema, decodeURIComponent(ref.substring(1)));
      return symbolForSchema(subSchema);
    } else {
      return null;
    }
  };

  var results = {};
  var i = 0;
  while (i < schemas.length) {
    var name = symbolForSchema(schemas[i]);
    results[name] = (0, _makeAst2.default)(schemas[i], refResolver);
    i++;
  }

  return _lodash2.default.map(results, function (ast, name) {
    return _ast2.default.Type(name, ast);
  });
};
exports.default = generateFlowAsts;