'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };
/* eslint-disable no-use-before-define */


var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _ast = require('./ast/ast.js');

var _ast2 = _interopRequireDefault(_ast);

var _simplify = require('./simplify.js');

var _simplify2 = _interopRequireDefault(_simplify);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var defaultResolver = function defaultResolver() {
  return null;
};

var makeAst = function makeAst(schema, refResolver) {
  if (schema.$ref) {
    // $FlowFixMe
    var resolved = refResolver(schema.$ref);
    if (resolved) {
      return _ast2.default.Literal(resolved);
    } else {
      return _ast2.default.Mixed;
    }
  } else if (schema.enum) {
    return _ast2.default.Union(_lodash2.default.map(schema.enum, _ast2.default.Exact));
  } else if (schema.oneOf) {
    return _ast2.default.Union(_lodash2.default.map(schema.oneOf, function (a) {
      return makeAst(a, refResolver);
    }));
  } else if (schema.anyOf) {
    return _ast2.default.Union(_lodash2.default.map(schema.anyOf, function (a) {
      return makeAst(a, refResolver);
    }));
  } else if (schema.allOf) {
    return _ast2.default.Intersection(_lodash2.default.map(schema.allOf, function (a) {
      return makeAst(a, refResolver);
    }));
  } else if (schema.type) {
    if (typeof schema.type === 'string') {
      switch (schema.type) {
        case 'boolean':
          return _ast2.default.Boolean;
        case 'null':
          return _ast2.default.Null;
        case 'integer':
          return _ast2.default.Number;
        case 'number':
          return _ast2.default.Number;
        case 'string':
          return _ast2.default.String;
        case 'object':
          return makeObjectAst(schema, refResolver);
        case 'array':
          return makeArrayAst(schema, refResolver);
        default:
          return _ast2.default.Mixed;
      }
    } else {
      // type can be either a string or an array of strings or schemas, so we
      // transform to an array of schemas and reduce later
      var typeArray = typeof schema.type === 'string' ? [schema.type] : schema.type;
      var schemaArray = _lodash2.default.map(typeArray, function (stringOrSchema) {
        if (typeof stringOrSchema === 'string') {
          return _extends({}, schema, {
            type: stringOrSchema
          });
        } else {
          return stringOrSchema;
        }
      });
      return _ast2.default.Union(_lodash2.default.map(schemaArray, function (u) {
        return makeAst(u, refResolver);
      }));
    }
  } else {
    return _ast2.default.Mixed;
  }
};

var makeObjectAst = function makeObjectAst(schema, refResolver) {
  // If properties, use emit a record. Otherwise, emit a map. We don't use
  // patternProperties because it's not expressable in flow.
  if (schema.properties) {
    var required = schema.required || [];
    return _ast2.default.Record(_lodash2.default.mapValues(schema.properties, function (subSchema, key) {
      var subType = makeAst(subSchema, refResolver);
      if (required.includes(key)) {
        return subType;
      } else {
        return _ast2.default.Optional(subType);
      }
    }));
  } else {
    var additionalProperties = schema.additionalProperties;

    if (additionalProperties && (typeof additionalProperties === 'undefined' ? 'undefined' : _typeof(additionalProperties)) === 'object') {
      return _ast2.default.Map(makeAst(additionalProperties, refResolver));
    } else {
      return _ast2.default.Map(_ast2.default.Mixed);
    }
  }
};

var makeArrayAst = function makeArrayAst(schema, refResolver) {
  if (schema.items) {
    if (Array.isArray(schema.items)) {
      return _ast2.default.Tuple(_lodash2.default.map(schema.items, function (t) {
        return makeAst(t, refResolver);
      }));
    } else {
      return _ast2.default.Array(makeAst(schema.items, refResolver));
    }
  } else {
    return _ast2.default.Array(_ast2.default.Mixed);
  }
};

exports.default = function (schema) {
  var refResolver = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultResolver;

  return (0, _simplify2.default)(makeAst(schema, refResolver));
};