'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _util = require('../util.js');

var _util2 = _interopRequireDefault(_util);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }
/* eslint-disable no-use-before-define */


var renderExact = function renderExact(value, depth) {
  return JSON.stringify(value);
};
var renderTuple = function renderTuple(ft, depth) {
  var contents = _lodash2.default.map(ft.children, function (child) {
    var rendered = render(child, depth + 1);
    return _util2.default.indent(rendered + ',', depth);
  });
  return ['['].concat(_toConsumableArray(contents), [_util2.default.indent(']', depth)]).join('\n');
};

var renderRecord = function renderRecord(ft, depth) {
  var contents = _lodash2.default.map(ft.fields, function (subtype, key) {
    if (subtype.type === 'optional') {
      var rendered = render(subtype.child, depth + 1);
      return _util2.default.indent(key + '?: ' + rendered + ',', depth + 1);
    } else {
      var _rendered = render(subtype, depth + 1);
      return _util2.default.indent(key + ': ' + _rendered + ',', depth + 1);
    }
  });
  return ['{'].concat(_toConsumableArray(contents), [_util2.default.indent('}', depth)]).join('\n');
};

var render = function render(ft) {
  var depth = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

  switch (ft.type) {
    case 'declaration':
      return 'declare type ' + ft.name + ' = ' + render(ft.value, depth) + ';';
    case 'type':
      return 'export type ' + ft.name + ' = ' + render(ft.value, depth) + ';';
    case 'literal':
      return ft.value;
    case 'exact':
      return renderExact(ft.value, depth);
    case 'mixed':
    case 'boolean':
    case 'null':
    case 'number':
    case 'string':
      return ft.type;
    case 'optional':
      return '?' + render(ft.child, depth);
    case 'array':
      return 'Array<' + render(ft.child, depth) + '>';
    case 'map':
      return '{[key: string]: ' + render(ft.child, depth) + '}';
    case 'tuple':
      return renderTuple(ft, depth);
    case 'record':
      return renderRecord(ft, depth);
    case 'union':
      return _lodash2.default.map(ft.children, function (child) {
        return render(child, depth);
      }).join(' | ');
    case 'intersection':
      return _lodash2.default.map(ft.children, function (child) {
        return render(child, depth);
      }).join(' & ');
    default:
      return '';
  }
};

exports.default = render;