'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _ast = require('./ast/ast.js');

var _ast2 = _interopRequireDefault(_ast);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable no-use-before-define */
var simplifyOptional = function simplifyOptional(ft) {
  var child = ft.child;

  var simplified = simplify(child);
  switch (child.type) {
    case 'optional':
      return simplified;
    default:
      return _ast2.default.Optional(simplified);
  }
};
var simplifyIU = function simplifyIU(Constructor, children) {
  var unique = _lodash2.default.uniqWith(_lodash2.default.map(children, simplify), _lodash2.default.isEqual);
  if (unique.length === 1) {
    return unique[0];
  } else {
    return Constructor(unique);
  }
};

var simplify = function simplify(ft) {
  switch (ft.type) {
    case 'declaration':
      return _ast2.default.Declaration(ft.name, simplify(ft.value));
    case 'optional':
      return simplifyOptional(ft);
    case 'array':
      return _ast2.default.Array(simplify(ft.child));
    case 'map':
      return _ast2.default.Map(simplify(ft.child));
    case 'tuple':
      return _ast2.default.Tuple(_lodash2.default.map(ft.children, simplify));
    case 'record':
      return _ast2.default.Record(_lodash2.default.mapValues(ft.fields, simplify));
    case 'union':
      return simplifyIU(_ast2.default.Union, ft.children);
    case 'intersection':
      return simplifyIU(_ast2.default.Intersection, ft.children);
    default:
      return ft;
  }
};

exports.default = simplify;